@@ -1,35 +1,3 @@
-.tri_options <- function(tri) {
-  c("min_fill", "min_rfill", "min_efill", "min_sfill", "min_rsfill", "min_esp", "min_sp", "min_nei", "minimal", "alpha")
-}
-
-check_params_compile <- function(tri, pmf_evidence, alpha, nodes, root_node) {
-
-  if (root_node != "") {
-    if (root_node %ni% nodes) {
-      stop("Invalid root_node", call. = FALSE)      
-    }
-  }
-  
-  if (!(tri %in% .tri_options())) {
-    stop("tri must be one of ", paste(.tri_options(), collapse = ", "), call. = FALSE)
-  }
-
-  if (tri %in% c("min_efill", "min_esp") && is.null(pmf_evidence)) {
-    stop(
-      "tri = " , tri,
-      "requires that pmf_evidence is specified",
-      call. = FALSE)
-  }
-
-  if (tri == "alpha")  {
-    if (is.null(alpha)) {
-      stop("tri = 'alpha' requires that the alpha parameter is specified", call. = FALSE)
-    }
-    if (!identical(sort(alpha), sort(nodes))) stop("The alpha parameter was not specified correctly")
-  }
-  invisible(NULL)
-}
-
 as_undirected_igraph <- function(g) igraph::as.undirected(g)
 
 parents_igraph <- function(g) {
@@ -74,11 +42,28 @@ graph_from_cpt_list <- function(x) {
   g
 }
 
+graph_from_pot_list <- function(x, cliques, nodes) {
+  g <- igraph::make_empty_graph(n = length(nodes), directed = FALSE)
+  g <- igraph::set_vertex_attr(g, "label", value = nodes)
+  g <- igraph::set_vertex_attr(g, "name", value = nodes)
+  edges <- lapply(cliques, function(clique) {
+    if (length(clique) == 1L) return(NULL)
+    t(utils::combn(clique, 2, simplify = TRUE))
+  })
+  edges <- Filter(neq_null, edges)
+  edges <- unique(do.call(rbind, edges))
+  for (k in 1:nrow(edges)) {
+    g <- igraph::add_edges(g, unname(edges[k, ]))
+  }
+  g
+}
+
+
 moralize_igraph <- function(g, parents) {
   g <- igraph::as.undirected(g)
   for (p in parents) {
     if (length(p) > 1) {
-      pairs <- utils::combn(p, 2,  simplify = FALSE)
+      pairs <- utils::combn(p, 2, simplify = FALSE)
       for (ps in pairs) {
         if (!igraph::are_adjacent(g, ps[1], ps[2])) {
           g <- g + igraph::edge(ps[1], ps[2]) 
