@@ -1,3 +1,16 @@
+ # min_rfill - DONE
+ # min_sp (cliques without log) - DONE
+
+
+# VARIANTS OF min_esp
+ # min_lsp (cliques and log)
+ # min_slsp (seps and log)
+
+# VARIANTS OF min_esp
+ # min_elsp (cliques and log)  # Just rename here!
+ # min_eslsp (seps and log)   
+
+
 # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 #                TRIANGULATION CONSTRUCTORS
 # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
@@ -25,6 +38,10 @@ new_base_triang <- function(x, cls) {
   )
 }
 
+new_min_nei_triang <- function(x) {
+  new_base_triang(x, cls = "min_nei_triang")
+}
+
 new_min_fill_triang <- function(x) {
   new_base_triang(x, cls = "min_fill_triang")
 }
@@ -40,41 +57,37 @@ new_min_efill_triang <- function(x, nlvls, pmf_evidence) {
   obj
 }
 
-new_min_elfill_triang <- function(x, nlvls, pmf_evidence) {
-  obj <- new_base_triang(x, cls = "min_elfill_triang")
-  obj$nlvls <- nlvls[dimnames(x)[[1]]]
-  obj$pmf_evidence <- pmf_evidence
-  obj
-}
-
-
-new_min_sfill_triang <- function(x, nlvls) {
-  obj <- new_base_triang(x, cls = "min_sfill_triang")
-  obj$nlvls <- nlvls[dimnames(x)[[1]]]
-  obj
-}
-
-new_min_rsfill_triang <- function(x, nlvls) {
-  obj <- new_base_triang(x, cls = "min_rsfill_triang")
-  obj$nlvls <- nlvls[dimnames(x)[[1]]]
-  obj
-}
-
-new_min_sp_triang <- function(x, nlvls) {
+new_min_sp_triang <- function(x, nlvls, clique_statespace = TRUE, log_statespace = FALSE) {
   obj <- new_base_triang(x, cls = "min_sp_triang")
   obj$nlvls <- nlvls[dimnames(x)[[1]]]
+  obj$clique_statespace <- clique_statespace
+  obj$log_statespace <- log_statespace
   obj
 }
 
-new_min_esp_triang <- function(x, nlvls, pmf_evidence) {
-  obj <- new_base_triang(x, cls = "min_esp_triang")
+new_min_ssp_triang <- function(x, nlvls, clique_statespace = FALSE, log_statespace = FALSE) {
+  new_min_sp_triang(x, nlvls, clique_statespace, log_statespace)
+}
+
+
+new_min_lsp_triang <- function(x, nlvls, clique_statespace = TRUE, log_statespace = TRUE) {
+  new_min_sp_triang(x, nlvls, clique_statespace, log_statespace)
+}
+
+new_min_lssp_triang <- function(x, nlvls, clique_statespace = FALSE, log_statespace = TRUE) {
+  new_min_sp_triang(x, nlvls, clique_statespace, log_statespace)
+}
+
+new_min_elsp_triang <- function(x, nlvls, pmf_evidence, clique_statespace = TRUE) {
+  obj <- new_base_triang(x, cls = "min_elsp_triang")
   obj$nlvls <- nlvls[dimnames(x)[[1]]]
   obj$pmf_evidence <- pmf_evidence
+  obj$clique_statespace <- clique_statespace
   obj
 }
 
-new_min_nei_triang <- function(x) {
-  new_base_triang(x, cls = "min_nei_triang")
+new_min_elssp_triang <- function(x, nlvls, pmf_evidence, clique_statespace = FALSE) {
+  new_min_elsp_triang(x, nlvls, pmf_evidence, clique_statespace = clique_statespace)
 }
 
 new_minimal_triang <- function(x) {
@@ -84,17 +97,36 @@ new_minimal_triang <- function(x) {
 }
 
 new_alpha_triang <- function(x, alpha) {
-  obj <- new_base_triang(x, cls = "min_sp_triang")
+  obj <- new_base_triang(x, cls = "alpha_triang")
   obj$alpha  <- alpha
   obj
 }
 
-# TODO: 
-# -----
-new_expected_triang <- function(x, data) {
-  obj <- new_base_triang(x, cls = "min_expected_triang")
-  obj$data <- data
-  obj
+new_triang <- function(tri, M, nlvls, pmf_evidence = NULL, alpha = NULL) {
+  # This one is used in compile from api_compile and triangulate from helper_triangulate
+  tri_obj <- switch(tri,
+
+    # min_fill based:
+    "min_fill"   = new_min_fill_triang(M),
+    "min_rfill"  = new_min_rfill_triang(M),
+
+    # min_sp based:
+    "min_sp"     = new_min_sp_triang(M, nlvls, TRUE, FALSE),           # clique statespace   
+    "min_ssp"    = new_min_ssp_triang(M, nlvls, FALSE, FALSE),         # separator statespace   
+    "min_lsp"    = new_min_lsp_triang(M, nlvls, TRUE, TRUE),           # log clique statespace   
+    "min_lssp"   = new_min_lssp_triang(M, nlvls, FALSE, TRUE),         # log separator statespace 
+    "min_elsp"   = new_min_elsp_triang(M, nlvls, pmf_evidence, TRUE),  # expected log clique statespace
+    "min_elssp"  = new_min_elssp_triang(M, nlvls, pmf_evidence, FALSE),# expected log separtor statespace
+
+    # others
+    "min_nei"    = new_min_nei_triang(M),
+    "minimal"    = new_minimal_triang(M),
+    "alpha"      = new_alpha_triang(M, alpha)
+  )
+
+  if (is.null(tri_obj)) stop(tri, "is not a valid triangulation method.")
+
+  tri_obj
 }
 
 # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
@@ -102,6 +134,7 @@ new_expected_triang <- function(x, data) {
 # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 new_node_to_eliminate <- function(obj) UseMethod("new_node_to_eliminate")
 
+
 new_node_to_eliminate.min_nei_triang <- function(obj) {
   x               <- obj$x
   new_node_idx    <- integer(0)
@@ -134,6 +167,7 @@ new_node_to_eliminate.min_nei_triang <- function(obj) {
   return(obj)
 }
 
+
 new_node_to_eliminate.min_fill_triang <- function(obj) {
   x <- obj$x
   
@@ -180,222 +214,66 @@ new_node_to_eliminate.min_rfill_triang <- function(obj) {
   return(obj)
 }
 
-new_node_to_eliminate.min_efill_triang <- function(obj) {
+new_node_to_eliminate.min_elsp_triang <- function(obj) {
+
+  # pmf_evidence (prob. for missingness): 0 -> always observed
+  # pmf_evidence (prob. for missingness): 1 -> never observed
+  
   x               <- obj$x
   new_node_idx    <- integer(0)
   current_nei_idx <- integer(0)
   nlvls           <- obj$nlvls
+  clique_statespace <- obj$clique_statespace
 
-  min_fills <- .map_dbl(1:ncol(x), function(k) {
+  simplicial_nodes_idx <- which(.map_dbl(1:ncol(x), function(k) {
     current_nei_idx_k <- which(x[, k] == 1L)
     all_edges <- length(current_nei_idx_k) * (length(current_nei_idx_k) - 1L) / 2
     existing_edges <- sum(x[current_nei_idx_k, current_nei_idx_k]) / 2L
     all_edges - existing_edges
-  })
+  }) == 0)
 
-  min_ <- min(min_fills)
-  candidate_nodes_idx <- which(min_fills == min_)
-
-  if (min_ == 0) {
-    new_node_idx    <- candidate_nodes_idx[1]
-    current_nei_idx <- which(x[, new_node_idx, drop = TRUE] == 1L)
+  if (neq_empt_int(simplicial_nodes_idx)) {
+    # Eliminate simplicial node
+    new_node_idx    <- simplicial_nodes_idx[1L]
+    current_nei_idx <- which(x[, new_node_idx] == 1L)
   } else {
 
+    nodes <- colnames(x)
     pmf <- obj$pmf_evidence
     evidence_nodes <- names(pmf)
-    nodes <- colnames(x)
-    expected_statespace <- Inf
 
-    for (k in candidate_nodes_idx) {
-      current_nei_idx_k   <- which(x[, k] == 1L)
-      family_k            <- nodes[c(k, current_nei_idx_k)]
-      evidence_nodes_k    <- intersect(evidence_nodes, family_k)
-      remaining_nodes_k   <- setdiff(family_k, evidence_nodes)
+    expected_statespace <- Inf
+    for (k in 1:ncol(x)) {
+
+      current_nei_idx_k <- which(x[, k] == 1L)
+      # Family is either the clique or separator depending on clique_statespace
+      family_k <- if (clique_statespace) nodes[c(k, current_nei_idx_k)] else nodes[current_nei_idx_k]
+      evidence_nodes_k <- intersect(evidence_nodes, family_k)
+      remaining_nodes_k <- setdiff(family_k, evidence_nodes)
+
+      sp_remaining_k <- nlvls[remaining_nodes_k]
       prod_sp_remaining_k <- prod(nlvls[remaining_nodes_k])
       
       if (neq_empt_chr(evidence_nodes_k)) {
         sp_evidence_k <- nlvls[evidence_nodes_k]
-        pmf_k         <- pmf[evidence_nodes_k]
-        pmf_k_lst     <- lapply(pmf_k, function(p) c(p, 1-p)) # pres/absc (0/1) = 1/2 here
-
-        # Calculate expected statespace
-        evidence_state_space <- expand.grid(replicate(length(pmf_k), c(1, 2), FALSE), stringsAsFactors = FALSE)
-        expected_sum_k <- sum(apply(evidence_state_space, 1L, function(state) {
-          state_prob <- .map_dbl(seq_along(pmf_k_lst), function(k) pmf_k_lst[[k]][state[k]])
-          prod(sp_evidence_k^c(state-1)) * prod(state_prob)
-        }))
-
-        expected_statespace_k <- expected_sum_k * prod_sp_remaining_k
+        pmf_k <- pmf[evidence_nodes_k]        
+        expected_statespace_k <- sum(log(sp_evidence_k) * pmf_k) + sum(log(sp_remaining_k))
         if (expected_statespace_k < expected_statespace) {
           expected_statespace <- expected_statespace_k
           current_nei_idx <- current_nei_idx_k
-          new_node_idx    <- k
+          new_node_idx <- k
         }
-      } else {
-        if (prod_sp_remaining_k < expected_statespace) {
-          expected_statespace <- prod_sp_remaining_k
-          current_nei_idx <- current_nei_idx_k
-          new_node_idx    <- k
-        }
-      }
-    }
-  }
-  
-  current_nei_mat     <- obj$x[current_nei_idx, current_nei_idx, drop = FALSE]
-  obj$current_nei_mat <- current_nei_mat
-  obj$current_nei_idx <- current_nei_idx
-  obj$is_nei_complete <- sum(current_nei_mat) == length(current_nei_idx) * (length(current_nei_idx) - 1)
-  obj$new_node_idx    <- new_node_idx
-  obj$nlvls           <- nlvls[-new_node_idx]
-
-  return(obj)
-}
-
-new_node_to_eliminate.min_elfill_triang <- function(obj) {
-  x               <- obj$x
-  new_node_idx    <- integer(0)
-  current_nei_idx <- integer(0)
-  nlvls           <- obj$nlvls
-
-  min_fills <- .map_dbl(1:ncol(x), function(k) {
-    current_nei_idx_k <- which(x[, k] == 1L)
-    all_edges <- length(current_nei_idx_k) * (length(current_nei_idx_k) - 1L) / 2
-    existing_edges <- sum(x[current_nei_idx_k, current_nei_idx_k]) / 2L
-    all_edges - existing_edges
-  })
-
-  min_ <- min(min_fills)
-  candidate_nodes_idx <- which(min_fills == min_)
-  
-  if (min_ == 0) {
-    new_node_idx    <- candidate_nodes_idx[1]
-    current_nei_idx <- which(x[, new_node_idx, drop = TRUE] == 1L)
-  } else {
-
-    pmf <- obj$pmf_evidence
-    evidence_nodes <- names(pmf)
-    nodes <- colnames(x)
-    expected_statespace <- Inf
-
-    for (k in candidate_nodes_idx) {
-      current_nei_idx_k   <- which(x[, k] == 1L)
-      # Family now equals the neibors (the new separator)!
-      family_k            <- nodes[current_nei_idx_k]
-      evidence_nodes_k    <- intersect(evidence_nodes, family_k)
-      remaining_nodes_k   <- setdiff(family_k, evidence_nodes)
-      sp_remaining_k      <- nlvls[remaining_nodes_k]
-      prod_sp_remaining_k <- prod(sp_remaining_k)
-      
-      if (neq_empt_chr(evidence_nodes_k)) {
-        sp_evidence_k <- nlvls[evidence_nodes_k]
-        pmf_k         <- pmf[evidence_nodes_k]
-        pmf_k_lst     <- lapply(pmf_k, function(p) c(p, 1-p)) # pres/absc (0/1) = 1/2 here
         
-        # Calculate expected statespace
-        # evidence_state_space <- expand.grid(replicate(length(pmf_k), c(1, 2), FALSE), stringsAsFactors = FALSE)
-        # expected_sum_k <- sum(apply(evidence_state_space, 1L, function(state) {
-        #   state_prob <- .map_dbl(seq_along(pmf_k_lst), function(k) pmf_k_lst[[k]][state[k]])
-        #   prod(sp_evidence_k^c(state-1)) * prod(state_prob)
-        # }))
-
-        expected_statespace_k <- sum(log(sp_evidence_k) * (1 - pmf_k)) + sum(log(sp_remaining_k))
+      } else {
+        expected_statespace_k <- prod_sp_remaining_k
         if (expected_statespace_k < expected_statespace) {
           expected_statespace <- expected_statespace_k
           current_nei_idx <- current_nei_idx_k
-          new_node_idx    <- k
-        }
-      } else {
-        if (prod_sp_remaining_k < expected_statespace) {
-          expected_statespace <- prod_sp_remaining_k
-          current_nei_idx <- current_nei_idx_k
-          new_node_idx    <- k
+          new_node_idx <- k          
         }
       }
     }
   }
-  
-  current_nei_mat     <- obj$x[current_nei_idx, current_nei_idx, drop = FALSE]
-  obj$current_nei_mat <- current_nei_mat
-  obj$current_nei_idx <- current_nei_idx
-  obj$is_nei_complete <- sum(current_nei_mat) == length(current_nei_idx) * (length(current_nei_idx) - 1)
-  obj$new_node_idx    <- new_node_idx
-  obj$nlvls           <- nlvls[-new_node_idx]
-
-  return(obj)
-}
-
-new_node_to_eliminate.min_sfill_triang <- function(obj) {
-  x               <- obj$x
-  nlvls           <- obj$nlvls
-  new_node_idx    <- integer(0)
-  current_nei_idx <- integer(0)
-  min_sp          <- Inf
-
-  min_fills <- .map_dbl(1:ncol(x), function(k) {
-    current_nei_idx_k <- which(x[, k] == 1L)
-    all_edges <- length(current_nei_idx_k) * (length(current_nei_idx_k) - 1L) / 2
-    existing_edges <- sum(x[current_nei_idx_k, current_nei_idx_k]) / 2L
-    all_edges - existing_edges
-  })
-
-  min_ <- min(min_fills)
-  candidate_nodes_idx <- which(min_fills == min_)
-
-  if (min_ == 0) {
-    new_node_idx    <- candidate_nodes_idx[1]
-    current_nei_idx <- which(x[, new_node_idx, drop = TRUE] == 1L)
-  } else {
-    for (k in candidate_nodes_idx) {
-      current_nei_idx_k <- which(x[, k] == 1L)
-      family_k <- c(k, current_nei_idx_k)
-      min_sp_k <- prod(nlvls[family_k])
-      if (min_sp_k < min_sp) {
-        min_sp          <- min_sp_k
-        new_node_idx    <- k
-        current_nei_idx <- current_nei_idx_k
-      }
-    }
-  }
-  
-  current_nei_mat     <- obj$x[current_nei_idx, current_nei_idx, drop = FALSE]
-  obj$current_nei_mat <- current_nei_mat
-  obj$current_nei_idx <- current_nei_idx
-  obj$is_nei_complete <- sum(current_nei_mat) == length(current_nei_idx) * (length(current_nei_idx) - 1)
-  obj$new_node_idx    <- new_node_idx
-  obj$nlvls           <- nlvls[-new_node_idx]
-
-  return(obj)
-}
-
-new_node_to_eliminate.min_rsfill_triang <- function(obj) {
-  x               <- obj$x
-  nlvls           <- obj$nlvls
-  new_node_idx    <- integer(0)
-  current_nei_idx <- integer(0)
-
-  min_fills <- .map_dbl(1:ncol(x), function(k) {
-    current_nei_idx_k <- which(x[, k] == 1L)
-    all_edges <- length(current_nei_idx_k) * (length(current_nei_idx_k) - 1L) / 2
-    existing_edges <- sum(x[current_nei_idx_k, current_nei_idx_k]) / 2L
-    all_edges - existing_edges
-  })
-
-  min_ <- min(min_fills)
-  candidate_nodes_idx <- which(min_fills == min_)
-
-  if (min_ == 0) {
-    new_node_idx    <- candidate_nodes_idx[1]
-    current_nei_idx <- which(x[, new_node_idx, drop = TRUE] == 1L)
-  } else {
-    all_min_sp <- .map_dbl(candidate_nodes_idx, function(k) {
-      current_nei_idx_k <- which(x[, k] == 1L)
-      family_k <- c(k, current_nei_idx_k)
-      prod(nlvls[family_k])
-    })
-    mins <- which(all_min_sp == min(all_min_sp))
-    new_node_idx <- candidate_nodes_idx[sample(mins, 1)]
-    current_nei_idx <- which(x[, new_node_idx, drop = TRUE] == 1L)
-  }
 
   current_nei_mat     <- obj$x[current_nei_idx, current_nei_idx, drop = FALSE]
   obj$current_nei_mat <- current_nei_mat
@@ -406,13 +284,16 @@ new_node_to_eliminate.min_rsfill_triang <- function(obj) {
 
   return(obj)
 }
-  
+
 new_node_to_eliminate.min_sp_triang <- function(obj) {
-  x               <- obj$x
-  nlvls           <- obj$nlvls
-  new_node_idx    <- integer(0)
-  current_nei_idx <- integer(0)
-  min_sp          <- Inf
+  x                 <- obj$x
+  nlvls             <- obj$nlvls
+  clique_statespace <- obj$clique_statespace
+  log_statespace    <- obj$log_statespace
+  
+  new_node_idx      <- integer(0)
+  current_nei_idx   <- integer(0)
+  min_sp            <- Inf
 
   simplicial_nodes_idx <- which(.map_dbl(1:ncol(x), function(k) {
     current_nei_idx_k <- which(x[, k] == 1L)
@@ -428,8 +309,9 @@ new_node_to_eliminate.min_sp_triang <- function(obj) {
   } else {
     for (k in 1:ncol(x)) {
       current_nei_idx_k <- which(x[, k] == 1L)
-      family_k <- c(k, current_nei_idx_k)
-      min_sp_k <- prod(nlvls[family_k])
+      # Either we eliminate on clique level or separator level
+      family_k <- if (clique_statespace) c(k, current_nei_idx_k) else current_nei_idx_k
+      min_sp_k <- if (log_statespace) log(prod(nlvls[family_k])) else prod(nlvls[family_k])
       if (min_sp_k < min_sp) {
         min_sp          <- min_sp_k
         new_node_idx    <- k
@@ -448,76 +330,6 @@ new_node_to_eliminate.min_sp_triang <- function(obj) {
   return(obj)
 }
 
-new_node_to_eliminate.min_esp_triang <- function(obj) {
-  x               <- obj$x
-  new_node_idx    <- integer(0)
-  current_nei_idx <- integer(0)
-  nlvls           <- obj$nlvls
-
-  simplicial_nodes_idx <- which(.map_dbl(1:ncol(x), function(k) {
-      current_nei_idx_k <- which(x[, k] == 1L)
-      all_edges <- length(current_nei_idx_k) * (length(current_nei_idx_k) - 1L) / 2
-      existing_edges <- sum(x[current_nei_idx_k, current_nei_idx_k]) / 2L
-      all_edges - existing_edges
-    }) == 0)
-
-  if (neq_empt_int(simplicial_nodes_idx)) {
-    # Eliminate simplicial node
-    new_node_idx    <- simplicial_nodes_idx[1L]
-    current_nei_idx <- current_nei_idx_k <- which(x[, new_node_idx] == 1L)
-  } else {
-    candidate_nodes_idx <- 1:ncol(x)
-    pmf <- obj$pmf_evidence
-    evidence_nodes <- names(pmf)
-    nodes <- colnames(x)
-    expected_statespace <- Inf
-
-    for (k in candidate_nodes_idx) {
-      current_nei_idx_k   <- which(x[, k] == 1L)
-      family_k            <- nodes[c(k, current_nei_idx_k)]
-      evidence_nodes_k    <- intersect(evidence_nodes, family_k)
-      remaining_nodes_k   <- setdiff(family_k, evidence_nodes)
-      prod_sp_remaining_k <- prod(nlvls[remaining_nodes_k])
-      
-      if (neq_empt_chr(evidence_nodes_k)) {
-        sp_evidence_k <- nlvls[evidence_nodes_k]
-        pmf_k         <- pmf[evidence_nodes_k]
-        pmf_k_lst     <- lapply(pmf_k, function(p) c(p, 1-p)) # pres/absc (0/1) = 1/2 here
-
-        # Calculate expected statespace
-        evidence_state_space <- expand.grid(replicate(length(pmf_k), c(1, 2), FALSE), stringsAsFactors = FALSE)
-        expected_sum_k <- sum(apply(evidence_state_space, 1L, function(state){
-          state_prob <- .map_dbl(seq_along(pmf_k_lst), function(k) pmf_k_lst[[k]][state[k]])
-          prod(sp_evidence_k^c(state-1)) * prod(state_prob)
-        }))
-
-        expected_statespace_k <- expected_sum_k * prod_sp_remaining_k
-        if (expected_statespace_k < expected_statespace) {
-          expected_statespace <- expected_statespace_k
-          current_nei_idx <- current_nei_idx_k
-          new_node_idx <- k
-        }
-      } else {
-        if (prod_sp_remaining_k < expected_statespace) {
-          expected_statespace <- prod_sp_remaining_k
-          current_nei_idx <- current_nei_idx_k
-          new_node_idx <- k
-        }
-      }
-    }
-  }
-  
-  current_nei_mat     <- obj$x[current_nei_idx, current_nei_idx, drop = FALSE]
-  obj$current_nei_mat <- current_nei_mat
-  obj$current_nei_idx <- current_nei_idx
-  obj$is_nei_complete <- sum(current_nei_mat) == length(current_nei_idx) * (length(current_nei_idx) - 1)
-  obj$new_node_idx    <- new_node_idx
-  obj$nlvls           <- nlvls[-new_node_idx]
-
-  return(obj)
-}
-
-
 new_node_to_eliminate.alpha_triang <- function(obj) {
   new_node_idx        <- match(obj$alpha[1L], colnames(obj$x))
   current_nei_idx     <- which(obj$x[, new_node_idx] == 1L)
